/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: ComputeStatistics.java,v 1.0 2003/08/04 08:07:19 diasz Exp $
 *
 * Date         Author  Changes
 */



/** Class used to parse files with timing values corresponding to the different server method calls used in the experiments, and the
* different segments from the request's path from the client to the server. For example one such file could be called a_method_SCL containing
* times spent in the client interceptor (SCL means Stay at CLient interceptor) by requests corresponding to the method call a_method
* performed during the experiments (for example the file could contain 200 such values)
* @author	<a href="mailto:diasz@ida.liu.se"> Diana Szentivanyi </a>
* @version $Revision: 1.0 $ $Date:2003/06/23$
*/


class DataSegm{
  long[] data;
  String segm;

  public DataSegm(long[] data,String segm){
    this.data=new long[data.length];
    System.arraycopy(data,0,this.data,0,data.length);
    this.segm=segm;
  }
}

class StatRec{
    int nr_samples;
    double st_dev;
    float mean;
    String segm;
    long sum_segm;

  public StatRec(double st_dev,float mean,String segm,int nr_samples,long sum_segm){
    this.st_dev=st_dev;
    this.mean=mean;
    this.segm=segm;
    this.nr_samples=nr_samples;
    this.sum_segm=sum_segm;
  }
}

 public class ComputeStatistics {

  public static void main(String args[]){
    java.io.RandomAccessFile to_read[],to_write;
    int i,j,k,l;
    String segm;
    float mean,mean_w_w,mean_rtt;
    double st_dev,st_dev_w_w;
    long data_rtt_w_w[],data_rtt[],data_tw[],data[],data_w_l[],sum_w,sum_rtt,sum_tw;
    java.util.Hashtable data_stat=new java.util.Hashtable();
    String path=args[0];
    int nr_largest=Integer.parseInt(args[2]);
    try{
      java.io.File file=new java.io.File(path+args[1]);
      String names[]=file.list(),op_name;
      java.util.Vector crt;
      Object an_elem;
      for(j=0;j<names.length;j++){
	  k=names[j].lastIndexOf('_');
	  segm=names[j].substring(k+1);
	  op_name=names[j].substring(0,k);
	  crt=(java.util.Vector)data_stat.get(op_name);
 	  if(crt==null){
	   crt=new java.util.Vector();
	   data_stat.put(op_name,crt);
	  }
	  data=Statistics.read_data_file(path+args[1]+names[j]);
	  data_w_l=Statistics.data_set_without_largest(data,nr_largest);
	  mean=Statistics.mean(data_w_l);
	  st_dev=Statistics.st_dev(data_w_l,mean);
	  crt.addElement(new StatRec(st_dev,mean,segm,data_w_l.length,Statistics.sum(data_w_l)));
	  l=crt.size();
      }


    java.util.Set entries;
    java.util.Iterator e_iter;
    java.util.Map.Entry elem;
    entries=data_stat.entrySet();
    e_iter=entries.iterator();
    long[] dif_w_w;
    int nr_samples;
    StatRec total,segm_stat;
    to_write=new java.io.RandomAccessFile(args[args.length-1],"rw");
    while(e_iter.hasNext()){
      elem=(java.util.Map.Entry)e_iter.next();
      op_name=(String)elem.getKey();

      crt=(java.util.Vector)elem.getValue();
      l=crt.size();
      if(l<5) continue;
      sum_rtt=0;
      nr_samples=0;
      sum_tw=0;
      for(i=0;i<l;i++){
	  segm_stat=(StatRec)crt.elementAt(i);
	  sum_rtt+=segm_stat.sum_segm;
	  if(segm_stat.segm.equals("TW"))
	      sum_tw=segm_stat.sum_segm;
	  if(i==l-1)
	      nr_samples=segm_stat.nr_samples;
      }

      mean_w_w=(float)(((float)(sum_rtt-sum_tw))/((float)nr_samples));
      to_write.writeBytes(op_name+"   ");
      mean_rtt=(float)((float)sum_rtt/(float)nr_samples);
      to_write.writeBytes(nr_samples+" "+(long)mean_rtt+" "+(long)mean_w_w+"\n");
      for(i=0;i<l;i++){
	segm_stat=(StatRec)crt.elementAt(i);
	to_write.writeBytes(op_name+" "+segm_stat.nr_samples+" "+segm_stat.segm+" "+(long)segm_stat.mean+" "+(long)segm_stat.st_dev+" "+(long)Statistics.proc(segm_stat.mean,mean_rtt)+"%\n");
      }
      to_write.writeBytes("\n\n");
    }
    }catch(java.lang.Exception e){e.printStackTrace();}
  }
}
