/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: Factory_name.java,v 1.0 2003/08/04 08:07:19 diasz Exp $
 *
 * Date         Author  Changes
 */


/* The local factory implementation for the application object creation*/

package a.package.name;
import org.omg.FT.*; /* Package name where the FT_CORBA infrastructure classes reside*/
import package.name.of.application.object.X;
import package.name.of.application.object.Y;
//.............................

/** Sample Object Factory class - HAS TO BE WRITTEN BY THE APPLICATION WRITER to incorporate initialization operations to
 * be transmitted to the Java Obejct Factory (in case this exists) for the specific application objects it has to create
 * It implements methods from the GenericFactory interface
 * @author <a href="mailto:diasz@ida.liu.se"> Diana Szentivanyi </a>
 * @version $Revision: 1.0 $ $Date:2003/06/23$
 */

class ReadThread extends Thread{
    java.io.InputStream is;
    public ReadThread(java.io.InputStream is){
	this.is=is;
    }

    public void run(){
	java.io.BufferedReader br=new java.io.BufferedReader(new java.io.InputStreamReader(is));
	String r;
	try{
	    while(true){
		r=br.readLine();
		if(r!=null)
		    System.out.println(r);
	    }
	}catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));}
    }
}


public class SampleApplicationObjectFactory extends org.omg.FT.GenericFactoryPOA{


    private class ObjIdStr{
	public long factcId;
	public org.omg.CORBA.Object object_ref;
	public java.lang.Process the_JVM;
	public boolean active_repl;
	public long obj_id;
	public int version;

	public ObjIdStr(long f_i,org.omg.CORBA.Object ref,java.lang.Process jvm,boolean active_r,long o_i,int vers){
	    factcId=f_i;
	    object_ref=ref;
	    the_JVM=jvm;
	    active_repl=active_r;
	    obj_id=o_i;
	    version=vers;
	}
    }

/** Class that incorporates information about a type identifier: the name of the class implementing the correponding interface, the vectors of
* constructor argument types and values (the values are those used when creating the replica in the factory), the method signature table of the interface; also,
* it contains methods to manipulate those informations: change the name of the implementing class (maybe also the argument list of the constructor called when
* creating an instance by the factory), remove a method from the interface (also form meth_sign_table), change parameters of a method, add parameters, remove
* parameters, change return type of the method, add an exception thrown by a method, remove an exception thrown by the method
*/

    private class ObjectItCanCreate{

	String class_name;
	java.util.Vector constr_arg_cl_names;
	java.util.Vector constr_arg_values;

	java.util.Vector method_sign_table;

	private java.lang.reflect.Constructor constr;
	private java.lang.Object[] ctr_arg_values;

	public ObjectItCanCreate(String class_name,java.util.Vector method_sign_table){
	    this.class_name=class_name;
	    this.method_sign_table=method_sign_table;
	    constr_arg_cl_names=new java.util.Vector();
	    constr_arg_values=new java.util.Vector();
	}



	public ObjectItCanCreate(String class_name,org.omg.FT.MethSignStr[] method_sign_table){
	    this.class_name=class_name;
	    int i;
	    this.method_sign_table=new java.util.Vector();
	    for(i=0;i<method_sign_table.length;i++)
		this.method_sign_table.addElement(method_sign_table[i]);
	    constr_arg_cl_names=new java.util.Vector();
	    constr_arg_values=new java.util.Vector();
	}

	public ObjectItCanCreate(String class_name,java.util.Vector method_sign_table,java.util.Vector constr_arg_cl_names,java.util.Vector constr_arg_values){
	    this.class_name=class_name;
	    this.method_sign_table=method_sign_table;
	    this.constr_arg_cl_names=constr_arg_cl_names;
	    this.constr_arg_values=constr_arg_values;
	    int size=constr_arg_cl_names.size(),i;
	    Class[] ctr_arg_classes=new Class[size];
	    ctr_arg_values=new java.lang.Object[size];
	    try{
		for(i=0;i<size;i++){
		    ctr_arg_classes[i]=Class.forName((String)constr_arg_cl_names.elementAt(i));
		    ctr_arg_values[i]=constr_arg_values.elementAt(i);
		}
		this.constr=Class.forName(this.class_name).getConstructor(ctr_arg_classes);
	    }catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));this.constr=null;}
	}



	public ObjectItCanCreate(String class_name,org.omg.FT.MethSignStr[] method_sign_table,java.util.Vector constr_arg_cl_names,java.util.Vector constr_arg_values){
	    this.class_name=class_name;
	    int i;
	    this.method_sign_table=new java.util.Vector();
	    for(i=0;i<method_sign_table.length;i++)
		this.method_sign_table.addElement(method_sign_table[i]);
	    this.constr_arg_cl_names=constr_arg_cl_names;
	    this.constr_arg_values=constr_arg_values;
	    int size=constr_arg_cl_names.size();
	    Class[] ctr_arg_classes=new Class[size];
	    ctr_arg_values=new java.lang.Object[size];
	    try{
		for(i=0;i<size;i++){
		    ctr_arg_classes[i]=Class.forName((String)constr_arg_cl_names.elementAt(i));
		    ctr_arg_values[i]=constr_arg_values.elementAt(i);
		}
		this.constr=Class.forName(this.class_name).getConstructor(ctr_arg_classes);
	    }catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));this.constr=null;}
	}

	public void change_class_name(String new_class_name){
	    class_name=new_class_name;
	    int size=constr_arg_cl_names.size(),i;
	    Class[] ctr_arg_classes=new Class[size];
	    ctr_arg_values=new java.lang.Object[size];
	    try{
		for(i=0;i<size;i++){
		    ctr_arg_classes[i]=Class.forName((String)constr_arg_cl_names.elementAt(i));
		    ctr_arg_values[i]=constr_arg_values.elementAt(i);
		}
		this.constr=Class.forName(this.class_name).getConstructor(ctr_arg_classes);
	    }catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));this.constr=null;}
	}

	public void change_class_name(String new_class_name,java.lang.Object[] new_arg_values){
	    class_name=new_class_name;
	    constr_arg_values=new java.util.Vector();
	    for(int i=0;i<new_arg_values.length;i++){
		constr_arg_values.addElement(new_arg_values[i]);
	    }
	    int size=constr_arg_cl_names.size(),i;
	    Class[] ctr_arg_classes=new Class[size];
	    ctr_arg_values=new java.lang.Object[size];
	    try{
		for(i=0;i<size;i++){
		    ctr_arg_classes[i]=Class.forName((String)constr_arg_cl_names.elementAt(i));
		    ctr_arg_values[i]=constr_arg_values.elementAt(i);
		}
		this.constr=Class.forName(this.class_name).getConstructor(ctr_arg_classes);
	    }catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));this.constr=null;}
	}

	public void change_class_name(String new_class_name,String[] new_arg_class_names,java.lang.Object[] new_arg_values){
	    class_name=new_class_name;
	    constr_arg_cl_names=new java.util.Vector();
	    constr_arg_values=new java.util.Vector();
	    int i;
	    for(i=0;i<new_arg_class_names.length;i++){
		constr_arg_cl_names.addElement(new_arg_class_names[i]);
		constr_arg_values.addElement(new_arg_values[i]);
	    }
	    int size=constr_arg_cl_names.size();
	    Class[] ctr_arg_classes=new Class[size];
	    ctr_arg_values=new java.lang.Object[size];
	    try{
		for(i=0;i<size;i++){
		    ctr_arg_classes[i]=Class.forName((String)constr_arg_cl_names.elementAt(i));
		    ctr_arg_values[i]=constr_arg_values.elementAt(i);
		}
		this.constr=Class.forName(this.class_name).getConstructor(ctr_arg_classes);
	    }catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));this.constr=null;}
	}

	public void remove_method(String method_name){
	    int i,l;
	    org.omg.FT.MethSignStr crt;
	    boolean found=false;
	    l=method_sign_table.size();
	    for(i=0;i<l;i++){
		crt=(org.omg.FT.MethSignStr)method_sign_table.elementAt(i);
		if(crt.name.equals(method_name)){
		    found=true;
		    break;
		}
	    }
	    if(found)
		method_sign_table.remove(i);
	}

	public void add_method(String method,org.omg.CORBA.TypeCode[] param_types,org.omg.CORBA.TypeCode[] except_types,int[] param_modes,char method_type){
	    /* Here it is supposed that this is a new method; otherwise use modify _method_* */
	    method_sign_table.addElement(new org.omg.FT.MethSignStr(method,method_type,param_types,param_modes,except_types));
	}

	public void add_method(org.omg.FT.MethSignStr sign){
	    method_sign_table.addElement(sign);
	}

	public void modify_method_add_param(String method,org.omg.CORBA.TypeCode param_type,int mode,int position){
	    int i,l,j,k;
	    org.omg.FT.MethSignStr crt;
	    org.omg.CORBA.TypeCode[] new_param_types;
	    int[] new_modes;
	    l=method_sign_table.size();
	    for(i=0;i<l;i++){
		crt=(org.omg.FT.MethSignStr)method_sign_table.elementAt(i);
		if(crt.name.equals(method)){
		    new_param_types=new org.omg.CORBA.TypeCode[crt.param_types.length+1];
		    new_modes=new int[crt.param_modes.length+1];
		    for(j=0;j<position;j++){
			new_param_types[j]=crt.param_types[j];
			new_modes[j]=crt.param_modes[j];
		    }
		    new_param_types[j]=param_type;
		    new_modes[j]=mode;
		    j++;
		    for(;j<new_param_types.length;j++)
			new_param_types[j]=crt.param_types[j-1];
		    k=position+1;
		    for(;k<new_modes.length;k++)
			new_modes[k]=crt.param_modes[k-1];
		    crt.param_types=new_param_types;
		    crt.param_modes=new_modes;
		    break;
		}
	    }
	}

	public void modify_method_add_except(String method,org.omg.CORBA.TypeCode except_type,int position){
	    int i,l,j;
	    org.omg.FT.MethSignStr crt;
	    org.omg.CORBA.TypeCode[] new_except_types;
	    l=method_sign_table.size();
	    for(i=0;i<l;i++){
		crt=(org.omg.FT.MethSignStr)method_sign_table.elementAt(i);
		if(crt.name.equals(method)){
		    new_except_types=new org.omg.CORBA.TypeCode[crt.except_types.length+1];
		    for(j=0;j<position;j++){
			new_except_types[j]=crt.except_types[j];
		    }
		    new_except_types[j]=except_type;
		    j++;
		    for(;j<new_except_types.length;j++)
			new_except_types[j]=crt.except_types[j-1];
		    crt.except_types=new_except_types;
		    break;
		}
	    }
	}

	public void modify_method_remove_param(String method,int position){
	    int i,l,j,k;
	    org.omg.FT.MethSignStr crt;
	    org.omg.CORBA.TypeCode[] new_param_types;
	    int[] new_modes;
	    l=method_sign_table.size();
	    for(i=0;i<l;i++){
		crt=(org.omg.FT.MethSignStr)method_sign_table.elementAt(i);
		if(crt.name.equals(method)){
		    if(crt.param_types.length<2) return;
		    new_param_types=new org.omg.CORBA.TypeCode[crt.param_types.length-1];
		    new_modes=new int[crt.param_modes.length-1];
		    for(j=0;j<position;j++){
			new_param_types[j]=crt.param_types[j];
			new_modes[j]=crt.param_modes[j];
		    }
		    for(;j<new_param_types.length;j++)
			new_param_types[j]=crt.param_types[j+1];
		    k=position;
		    for(;k<new_modes.length;k++)
			new_modes[k]=crt.param_modes[k+1];
		    crt.param_types=new_param_types;
		    crt.param_modes=new_modes;
		    break;
		}
	    }
	}
	public void modify_method_remove_except(String method,int position){
	    int i,l,j;
	    org.omg.FT.MethSignStr crt;
	    org.omg.CORBA.TypeCode[] new_except_types;

	    l=method_sign_table.size();
	    for(i=0;i<l;i++){
		crt=(org.omg.FT.MethSignStr)method_sign_table.elementAt(i);
		if(crt.name.equals(method)){
		    if(crt.except_types.length<1) return;
		    new_except_types=new org.omg.CORBA.TypeCode[crt.except_types.length-1];

		    for(j=0;j<position;j++){
			new_except_types[j]=crt.except_types[j];
		    }
		    for(;j<new_except_types.length;j++)
			new_except_types[j]=crt.except_types[j+1];

		    crt.except_types=new_except_types;
		    break;
		}
	    }
	}
	public void modify_method_modify_param(String method,org.omg.CORBA.TypeCode new_param_type,int new_mode,int position){
	    int i,l,j;
	    org.omg.FT.MethSignStr crt;

	    l=method_sign_table.size();
	    for(i=0;i<l;i++){
		crt=(org.omg.FT.MethSignStr)method_sign_table.elementAt(i);
		if(crt.name.equals(method)){
		    crt.param_types[position]=new_param_type;
		    crt.param_modes[position]=new_mode;
		    break;
		}
	    }
	}
	public void modify_method_modify_except(String method,org.omg.CORBA.TypeCode new_except_type,int position){
	    int i,l,j;
	    org.omg.FT.MethSignStr crt;

	    l=method_sign_table.size();
	    for(i=0;i<l;i++){
		crt=(org.omg.FT.MethSignStr)method_sign_table.elementAt(i);
		if(crt.name.equals(method)){
		    crt.except_types[position]=new_except_type;
		    break;
		}
	    }
	}

	public void modify_method_modify_type(String method,char new_type){
	    int i,l,j;
	    org.omg.FT.MethSignStr crt;
	    l=method_sign_table.size();
	    for(i=0;i<l;i++){
		crt=(org.omg.FT.MethSignStr)method_sign_table.elementAt(i);
		if(crt.name.equals(method))
		    crt.type=new_type;
	    }
	}

	public java.lang.Object create_instance() throws org.omg.FT.NoFactory{
	    if(constr==null) throw new org.omg.FT.NoFactory();
	    java.lang.Object toRet;
	    try{
		toRet=constr.newInstance(ctr_arg_values);
	    }catch(java.lang.Exception e){e.printStackTrace(new java.io.PrintStream(System.out));throw new org.omg.FT.NoFactory();}
	    return toRet;
	}

	public org.omg.FT.MethSignStr[] get_meth_sign_table(){
	    org.omg.FT.MethSignStr[] toRet=new org.omg.FT.MethSignStr[method_sign_table.size()];
	    toRet=(org.omg.FT.MethSignStr[])method_sign_table.toArray(toRet);
	    return toRet;
	}
    }

    java.util.Vector theObjects;
    long obj_index;
    String location=null,ftDom=null,classpath;
    org.omg.CORBA.ORB orb;
    boolean debug;

    java.util.Vector method_signature_tables,type_ids_vector;

    java.util.Hashtable objects_it_can_create;

    public SampleApplicationObjectFactory(Boolean debug,org.omg.CORBA.ORB _o,String location,String ftDom,String classpath/*,org.omg.PortableServer.POA _p*/){
	theObjects=new java.util.Vector();
	this.classpath=classpath;
	obj_index=0;
	this.debug=debug.booleanValue();
	orb=_o;
	this.location=location;
	this.ftDom=ftDom;
	method_signature_tables=new java.util.Vector();
	objects_it_can_create=new java.util.Hashtable();
	/* Initial type_id_adding*/
	java.util.Vector ctr_cl_names=new java.util.Vector(),ctr_arg_values=new java.util.Vector();

    /* Here, an initial set of type identifiers for which this factory can create objects, should be given.
    The API defined above can be used to add objects of type ObjectItCanCreate to the hashtable.
    Of course, the data about objects to be created (such as method signature table) can be created in a hardcoded
    manner, not necessarily using the dynamic API
    For example if the Factory should be able to create instances of a class corresponding to the type identifier
    IDL:a/b/InterfaceNameC:1.0, where the implementing class is a.b.CImpl, and the constructor arguments that have to
    be ued when creating this instance are 12 (an integer), true (a boolean) and "aaa" (a String) then the code that has to be
    written also using the API of ObjectItCanCreate looks as follows:

       java.util.Vector ctr_cl_names=new java.util.Vector(),ctr_arg_values=new java.util.Vector();

	   ctr_cl_names.addElement("int");
	   ctr_cl_names.addElement("boolean");
	   ctr_cl_names.addElement("String");

	   ctr_arg_values.addElement(new Integer(12));
	   ctr_arg_values.addElement(new Boolean(true));
	   ctr_arg_values.addElement(new String("aaa"));

	   ObjectItCanCreate first_type_id=new ObjectItCanCreate("a.b.CImpl",meth_sign_table,ctr_cl_names,ctr_arg_values);
	   objects_it_can_create.put("IDL:a/b/InterfaceNameC:1.0",first_type_id);

	   Note that meth_sign_table has to be obtained somehow by the application writer (e.g. via reflection, to find out method names and parameter
	   types); also, method types (update or read-only) have to be given

	   If another type id also has to be added then the same procedure can be performed again

       If new type-ids have to be added after the factory is running (not via hardcoding in the constructor) then the methods of the devised API can be used:
        add_new_type_id - used to add a new type_id info
        modify_class_name_ti - used if the implementing class corresponding to a certain type id changed (name)

	   When the factory is asked to create an instance of a class corresponding to a certain type_id, a different Java class is started in a different
	   JVM and this is in fact responsible for the creation of the CORBA object; the method signature table structure will be transmitted immediately
	   after the CORBA object's creation to the ORB in which this is running. The data about constructor argument values and types can be transmitted as command
	   line arguments (in the command string constructed later), or written in a file, and the file name tansmitted to the other JVM in the command string. In the
	   present implementation of the two classes, these prcatices are not used. Rather,in the JavaObjectFactory class when it is called the initialization of the
	   class corresponding to the type_id is done in a hardcoded  way.
	   OF COURSE, BOTH the application object factory and the Java Object Factory can be modified, as well as collapsed in one single class. The latter one
	   can happen if there is NO need to run every replica (from different groups) running on the same host, in different Java Virtual Machines!! In such a
	   case, the factory JVM will run all application replicas perhaps on different ORBs, but in the same JVM (UNIX process)!!!!
	*/
    }

    private void remove_old_version(long object_id,int version){
	int i,l;
	boolean found=false;
	ObjIdStr crt=null;

	l=theObjects.size();

	for(i=0;i<l;i++){
	    crt=(ObjIdStr)theObjects.elementAt(i);
	    if(crt.obj_id==object_id&&crt.version==version){
		found=true;
		break;
	    }
	}
	if(found){
	    theObjects.remove(i);
		crt.the_JVM.destroy();
	}
    }

    private org.omg.CORBA.Object obj_ref_vers(long object_id,int version){
	int i,l;
	boolean found=false;
	ObjIdStr crt=null;

	l=theObjects.size();

	for(i=0;i<l;i++){
	    crt=(ObjIdStr)theObjects.elementAt(i);
	    if(crt.obj_id==object_id&&crt.version==version){
		found=true;
		break;
	    }
	}
	if(found)
	    return crt.object_ref;
	return null;
    }

    private ObjIdStr search_creation(org.omg.CORBA.Any factory_creation_id){
	long f_c_i;
	int l,i;
	ObjIdStr crt;

	f_c_i=factory_creation_id.extract_longlong();
	l=theObjects.size();

	for(i=0;i<l;i++){
	    crt=(ObjIdStr)theObjects.elementAt(i);
	    if(crt.factcId==f_c_i)
		return crt;
	}
	return null;
    }

    /** Method used to add a new type id together with information about it given as call parameters
    * @param	type_id the type identifier to be added
    * 			class_name the name of the class implementing the corresponding interface
    *			new_arg_class_n the constructor arguments' class names
    *			new_arg_values the constructor arguments' values
    *			methods the method signature table of the corresponding interface
    */

    public void add_new_type_id(String type_id,String class_name,String[] new_arg_class_n,java.lang.Object[] new_arg_values,org.omg.FT.MethSignStr[] methods){
	java.util.Vector vector_cl_n,vector_arg_val;

	vector_cl_n=new java.util.Vector();
	vector_arg_val=new java.util.Vector();

	for(int i=0;i<new_arg_class_n.length;i++){
	    vector_cl_n.addElement(new_arg_class_n[i]);
	    vector_arg_val.addElement(new_arg_values[i]);
	}
	objects_it_can_create.put(type_id,new ObjectItCanCreate(class_name,methods,vector_cl_n,vector_arg_val));
    }

/** Method used to remove a type_id from the ones that this factory can create objects for*/

    public void remove_type_id(String type_id){
	objects_it_can_create.remove(type_id);
    }

/** Method used to modify the class name implementing the interface corresponding to a certain type identifier*/

    public void modify_class_name_ti(String type_id,String new_class_name){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.change_class_name(new_class_name);
    }

/** Method used to modify the class name implementing the interface corresponding to a certain type identifier, as well as the constructor parameter list*/

    public void modify_class_name_ti_ctr(String type_id,String new_class_name,String[] class_names,java.lang.Object[] arg_values){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.change_class_name(new_class_name,class_names,arg_values);
    }

/** Method used to add a new method signature to the signature table*/

    public void add_new_method_type_id_s(String type_id,org.omg.FT.MethSignStr the_method){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.add_method(the_method);
    }

    public void add_new_method_type_id_d(String type_id,String method_n,org.omg.CORBA.TypeCode[] par_types,org.omg.CORBA.TypeCode[] exc_types,int[] modes,char method_t){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.add_method(method_n,par_types,exc_types,modes,method_t);
    }

    public void remove_method_type_id(String type_id,String the_method){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.remove_method(the_method);
    }

    public void modify_meth_type_id_add_param(String type_id,String the_method,org.omg.CORBA.TypeCode par_type,int mode,int pos){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.modify_method_add_param(the_method,par_type,mode,pos);
    }

      public void modify_meth_type_id_add_exc(String type_id,String the_method,org.omg.CORBA.TypeCode exc_type,int pos){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.modify_method_add_except(the_method,exc_type,pos);
    }


    public void modify_meth_type_id_remove_param(String type_id,String the_method,int pos){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.modify_method_remove_param(the_method,pos);
    }

      public void modify_meth_type_id_remove_exc(String type_id,String the_method,int pos){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.modify_method_remove_except(the_method,pos);
    }


      public void modify_meth_type_id_modify_param(String type_id,String the_method,org.omg.CORBA.TypeCode par_type,int mode,int pos){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.modify_method_modify_param(the_method,par_type,mode,pos);
    }

      public void modify_meth_type_id_modify_exc(String type_id,String the_method,org.omg.CORBA.TypeCode exc_type,int pos){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.modify_method_modify_except(the_method,exc_type,pos);
      }

    public void modify_meth_type_id_modify_type(String type_id,String the_method,char new_type){
	ObjectItCanCreate the_obj;

	the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	if(the_obj==null) return;
	the_obj.modify_method_modify_type(the_method,new_type);
      }

    public org.omg.FT.MethSignStr[] return_meth_sign_table(String type_id){
		the_obj=(ObjectItCanCreate)objects_it_can_create.get(type_id);
	    if(the_obj==null) return null;
	    return the_obj.get_meth_sign_table();
	}


	/** Method called by the Replication Manager on every factory object residing on hosts where it wants to create replica group members
	 * In the present implementation, the factory forks a new process (java virtual machine) where it runs another java class that will do the
	 * actual creation of the class instance corresponding to the requested type_id. The crieria array is based on what the application writer
	 * requested: replication style, if this replica is a primary or a backup, if it is active of not (e.g. in warm passive repl. backups are activated
	 * at creation). This factory object finds out information about the method signature table of the replica object (either by using the API provided
	 * for dynamic managing of type_id information, or by using ome hardcoded way of access).
	 * @ param	type_id the type identifier of the CORBA object for which the replica will be created by this factory
	 *			the_criteria a set of properties determining how the replica will be created
	 *			factory_creation_id an identifier of this replica, used later when deleting it
	 * @return	the CORBA object reference for this replica; the reference is used by the Replication Manager to assemble the collective group reference
	*/


    public org.omg.CORBA.Object create_object(java.lang.String type_id, org.omg.FT.Property[] the_criteria, org.omg.CORBA.AnyHolder factory_creation_id) throws org.omg.FT.NoFactory{

	org.omg.CORBA.Any theIds[],any;
	int i,match,type=-1,version=-1;
	boolean found_oi=false,activate=false,active=false,stateless=false,warm_passive=false,join=false,create_gw=false,is_backup=false,checkp=false;
	byte[] oid=new byte[0];
	String[] oid_b;
	org.omg.PortableServer.POA new_poa,r_poa=null,gw_poa=null;
	org.omg.CORBA.ORB o_orb,gw_orb=null;
	String client_id,real_client_id;
	org.omg.CORBA.Object toReturn,obj_ref;

	Runtime the_launcher=Runtime.getRuntime();

	String CLASSPATH=classpath;
	String command="java -Xbootclasspath:"+CLASSPATH+" package.name.where.the.separate.JavaVMcreator.is.SeparateJavaVMCreatorClassName",command_gw=new String(command);

	org.omg.CORBA.NVList dyn_arguments;
	org.omg.CORBA.ExceptionList dyn_exceptions=orb.create_exception_list();
	org.omg.CORBA.NamedValue dyn_result=orb.create_named_value("",orb.create_any(),-1);
	org.openorb.CORBA.dii.Request dyn_request;
	org.omg.CORBA.Any argument;
	int flags=org.omg.CORBA.ARG_IN.value,life=-1;
	long oid_l=-1,checkp_int=-1;

	try{

	    for(i=0;i<the_criteria.length;i++){
		if(the_criteria[i].nam[0].id.equals("org.omg.ft.Version")){
		    version=the_criteria[i].val.extract_long();
		    if(found_oi){
			toReturn=obj_ref_vers(oid_l,version);
			if(toReturn!=null)
			    return toReturn;
		    }
		}
		else
			    if(the_criteria[i].nam[0].id.equals("org.omg.ft.LifeLength"))
				life=the_criteria[i].val.extract_long();
			    else
				if(the_criteria[i].nam[0].id.equals("org.omg.ft.ActivateIt"))
				    activate=true;
				else
				    if(the_criteria[i].nam[0].id.equals("org.omg.ft.ObjectId")){
					oid=org.omg.PortableServer.ObjectIdHelper.extract(the_criteria[i].val);
					found_oi=true;
					oid_l=org.omg.FT.Utils.makeByteId(oid);
				    }
				    else
					if(the_criteria[i].nam[0].id.equals("org.omg.ft.Active"))
					    active=true;
					else
					    if(the_criteria[i].nam[0].id.equals("org.omg.ft.FirstFactory"))
						create_gw=true;
					    else
						if(the_criteria[i].nam[0].id.equals("org.omg.ft.Join"))
						    join=true;
						else
						    if(the_criteria[i].nam[0].id.equals("org.omg.ft.NoLogging"))
							stateless=true;
						    else
							if(the_criteria[i].nam[0].id.equals("org.omg.ft.WarmPassive"))
							    warm_passive=true;
							else
							    if(the_criteria[i].nam[0].id.equals("org.omg.ft.Backup"))
								is_backup=true;
							    else
								if(the_criteria[i].nam[0].id.equals("org.omg.ft.CheckpInterval")){
								    checkp=true;
								    checkp_int=the_criteria[i].val.extract_longlong();
								}
								else
								    if(the_criteria[i].nam[0].id.equals("org.omg.FT.CheckpTrigger"))
									checkp=true;
	    }

	    command+="0 "+location+" "+ftDom+" "+type_id;
	    if(debug) command+=" 1";
	    else
		command+=" 0";
	    if(create_gw)
		command_gw+="1 "+location+" "+ftDom+" "+type_id;
	    if(found_oi){
		if(oid.length==8){
		    command+=" 1";
		    if(create_gw)
			command_gw+=" 1";
		}
		else{
		    command+=" 2";
		    command+=" "+Integer.toString(oid.length);
		    if(create_gw){
			command_gw+=" 2";
			command_gw+=" "+Integer.toString(oid.length);
		    }
		}
		oid_b=org.omg.FT.Utils.makeByteArgs(oid);
		for(i=0;i<oid_b.length;i++){
		    command+=" "+oid_b[i];
		    if(create_gw)
			command_gw+=" "+oid_b[i];
		}
	    }
	    else{
		command+=" 0";
		if(create_gw)
		    command_gw+=" 0";
	    }
	    if(activate)
		command+=" 1";
	    else
		command+=" 0";
	    if(active)
		command+=" 1";
	    else
		command+=" 0";
	    if(join)
		command+=" 1";
	    else
		command+=" 0";
	    if(stateless)
		command+=" 1";
	    else
		command+=" 0";
	    if(warm_passive)
		command+=" 1";
	    else
		command+=" 0";
	    if(is_backup)
		command+=" 1";
	    else
		command+=" 0";
	    if(life!=-1)
		command+=" l "+Integer.toString(life);
	    else
		command+=" lm1";

	    java.lang.Process this_replica,this_gateway;
	    java.io.InputStream is;
	    java.io.BufferedReader br;
	    org.omg.CORBA.Object target=null,target_gw=null;
	    org.omg.FT.MethSignStr[] methods;

	    this_replica=the_launcher.exec(command);
	    is=this_replica.getInputStream();

	    br=new java.io.BufferedReader(new java.io.InputStreamReader(is));

	    String objectRefStr;
	    boolean found=false;
	    while(true){
		objectRefStr=br.readLine();
		if(objectRefStr!=null) {
		    if(objectRefStr.startsWith("%%REF IS%%")){
			objectRefStr=objectRefStr.substring(10);
			found=true;
			new ReadThread(is).start();
			break;
		    }
		}
	    }
	    if(found)
		target=orb.string_to_object(objectRefStr);
	    if(create_gw){
		this_gateway=the_launcher.exec(command_gw);
		is=this_gateway.getInputStream();
		br=new java.io.BufferedReader(new java.io.InputStreamReader(is));
		found=false;
		while(true){
		    objectRefStr=br.readLine();
		    if(objectRefStr!=null) {
			if(objectRefStr.startsWith("%%REF IS%%")){
			    objectRefStr=objectRefStr.substring(10);
			    if(!objectRefStr.equals("null"))
				found=true;
			    new ReadThread(is).start();
			    break;
			}
		    }
		}
		if(found)
		    target_gw=orb.string_to_object(objectRefStr);
	    }

		methods=return_meth_sign_table(type_id);

	    org.omg.CORBA.Any methods_wr=orb.create_any();
	    org.omg.FT.MethSignStrsHelper.insert(methods_wr,methods);
	    if(create_gw){
		if(target_gw!=null&&target!=null){
		    org.omg.IOP.IOR obj_ior=Utils.get_ior(target),gw_ior=Utils.get_ior(target_gw);
		    org.omg.IOP.TaggedProfile profs[];
		    profs=new org.omg.IOP.TaggedProfile[2];
		    profs[0]=obj_ior.profiles[0];
		    profs[1]=gw_ior.profiles[0];
		    obj_ref=new org.openorb.CORBA.ObjectStub(orb,new org.omg.IOP.IOR(type_id,profs));
		}
		else
		    obj_ref=null;
	    }
	    else
		obj_ref=target;
	    dyn_arguments=orb.create_list(0);
	    argument=orb.create_any();
	    argument.insert_any(methods_wr);
	    dyn_arguments.add_value("param0",argument,flags);
	    dyn_request=new org.openorb.CORBA.dii.Request(target,"set_method_table",dyn_arguments,dyn_result,orb.create_environment(),dyn_exceptions,orb.create_context_list(),orb);
	    dyn_request.send_oneway();
	    if(create_gw){
		dyn_request=new org.openorb.CORBA.dii.Request(target_gw,"set_method_table",dyn_arguments,dyn_result,orb.create_environment(),dyn_exceptions,orb.create_context_list(),orb);
		dyn_request.send_oneway();
	    }

	    theObjects.addElement(new ObjIdStr(obj_index,obj_ref,this_replica,active,oid_l,version));
	    factory_creation_id.value=orb.create_any();
	    factory_creation_id.value.insert_longlong(obj_index);

	    obj_index++;
	}catch(Exception e){e.printStackTrace(new java.io.PrintStream(System.out));return null;}
	return obj_ref;
    }

    public void install_gateway(org.omg.CORBA.Any a,org.omg.CORBA.Object gw){}

    public void activate_object(org.omg.CORBA.Any f_c_i,long g_i,boolean f_t){}

    public void delete_object(org.omg.CORBA.Any factory_creation_id) throws org.omg.FT.ObjectNotFound{

	ObjIdStr crt=null;
	crt=search_creation(factory_creation_id);
	if(crt==null)  {System.out.println("Object with id "+factory_creation_id.extract_longlong()+" was not found");throw new org.omg.FT.ObjectNotFound();}
	theObjects.remove(crt);
	try{
	    crt.the_JVM.destroy();
	}catch(Exception e){e.printStackTrace(new java.io.PrintStream(System.out));}

    }

    public int get_load(){
	return theObjects.size();
    }
}
