/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: Name.java,v 1.0 2003/08/04 08:07:19 diasz Exp $
 *
 * Date         Author  Changes
 */

import org.omg.CosNaming.*;
import org.omg.FT.*;
/** Sample class used by the application writer to create groups
 * @author <a href="mailto:diasz@ida.liu.se"> Diana Szentivanyi </a>
 * @version $Revision: 1.0 $ $Date:2003/06/23$
 */

public class SampleServerLauncher{
    /** The command line arguments when running the java ServerLauncher command are:
      * the fault tolerance domain
      * the fault monitoring granularity
      * the file name where the Object Group Referenceobtained from the Replication Manager will be published
      * the fault monitoring interval
      * the replication style
      * the minimum number of replicas
      * the initial number of replicas
      * the checkpointing interval (0 if no periodic checkpointing)
      * the IDL type of the objects in the object group
      * a value saying whether to register the group reference in the name service as well
      * the number of requests after whic the server will be forced to fail (mainly useful when performing failover experiments)
      * the fault detection timeout
      *
      *
      * The code of the server launcher can be rewritten to have different parameters
      */

    public static void main(String args[]){
	if(args.length<11){
	    /*args[0]=ftDom,args[1]=fault_mon_gran,args[3]=ref_file,args[4]=repl_style,
	      args[5]=min_nr_replicas,args[6]=initial_nr_repl,args[7]=checkpoint_interval (0 is no
	      checkpointing),args[8]=IDL type, args[9]=put the ref in name service or not*/
	    System.out.println("Wrong usage: please put 11 arguments");
	    System.exit(0);
	}


	try{
	    org.omg.CORBA.Object rm_o,group_ref,rootC_o,domC_o,factC_o;
	    org.omg.FT.ReplicationManager rm;
	    org.omg.CORBA.AnyHolder factory_cr_id=new org.omg.CORBA.AnyHolder();
	    java.util.Properties props=System.getProperties();
	    Property[] criteria=new Property[0],type_pr;
	    NamingContext rootC,ftDomC,hostC,factC;
	    String hosts[];
	    int i,j;

	    hosts=Utils.parse_ft_dom_file(args[0]);

	    if(hosts==null||hosts.length==0) {System.out.println("No hosts in ft. domain "+args[0]);return;}

	   NameComponent ftDomNCs[]=new NameComponent[1],hostNCs[]=new NameComponent[1],factNC=new NameComponent("factory","text"),factNCs[]={factNC};

	    props.put("org.omg.CORBA.ORBClass","org.openorb.CORBA.ORB");
	    props.put("org.omg.CORBA.ORBSingletonClass","org.openorb.CORBA.ORBSingleton");
	    org.omg.CORBA.ORB orb=org.omg.CORBA.ORB.init(args,props);
	    org.omg.CORBA.Any any=orb.create_any();

	    rm_o=orb.resolve_initial_references("ReplicationManager");

	    rm=ReplicationManagerHelper.narrow(rm_o);


	    FactoryInfo[] fact_infos;

	    GenericFactory gf;

	    rootC_o=orb.resolve_initial_references("NameService");
	    rootC=NamingContextHelper.narrow(rootC_o);
	    ftDomNCs[0]=new NameComponent(args[0],"directory");
	    ftDomC=Utils.resolve_n(rootC,ftDomNCs);

	    if(ftDomC==null) {System.out.println("No "+args[0]+" context");return;}

	    java.util.Vector infos=new java.util.Vector();
	    boolean error;
	    j=0;
	    for(i=0;i<hosts.length;i++){
		error=false;
		System.out.println(hosts[i]);
		hostNCs[0]=new NameComponent(hosts[i],"directory");
		hostC=Utils.resolve_n(ftDomC,hostNCs);
		gf=null;
		if(hostC!=null){
		    try{
			gf=GenericFactoryHelper.narrow(hostC.resolve(factNCs));
		    }catch(java.lang.Exception e){e.printStackTrace();error=true;}
		    /* by now, the location field in FactoryInfo is the location given by hosts[i]*/
		    if(!error){
			NameComponent[] location=new NameComponent[1];
			location[0]=new NameComponent(hosts[i],"loc");
			infos.addElement(new FactoryInfo(gf,location,new Property[0]));
		    }
		}
	    }
	    fact_infos=new FactoryInfo[infos.size()];
	    fact_infos=(FactoryInfo[])infos.toArray(fact_infos);
	    if(args[7].charAt(0)=='0')
		type_pr=new Property[9];
	    else
		type_pr=new Property[11];

	    NameComponent prop_n,prop_ns[];

	    prop_n=new NameComponent("org.omg.ft.LifeLength","string");
	    prop_ns=new NameComponent[1];
	    prop_ns[0]=prop_n;
	    any=orb.create_any();
	    any.insert_long(Integer.parseInt(args[10]));
	    type_pr[0]=new Property(prop_ns,any);

	    prop_n=new NameComponent("org.omg.ft.ReplicationStyle","string");
	    prop_ns=new NameComponent[1];
	    prop_ns[0]=prop_n;
	    any=orb.create_any();
	    any.insert_long(Integer.parseInt(args[4]));
	    type_pr[1]=new Property(prop_ns,any);

	    prop_n=new NameComponent("org.omg.ft.InitialNumberReplicas","string");
	    prop_ns=new NameComponent[1];
	    prop_ns[0]=prop_n;
	    any=orb.create_any();
	    any=orb.create_any();
	    any.insert_long(Integer.parseInt(args[6]));
	    type_pr[2]=new Property(prop_ns,any);

	    prop_n=new NameComponent("org.omg.ft.MinimumNumberReplicas","string");
	    prop_ns=new NameComponent[1];
	    prop_ns[0]=prop_n;
	    any=orb.create_any();
	    any.insert_long(Integer.parseInt(args[5]));
	    type_pr[3]=new Property(prop_ns,any);

	    prop_n=new NameComponent("org.omg.ft.MembershipStyle","string");
	    prop_ns=new NameComponent[1];
	    prop_ns[0]=prop_n;
	    any=orb.create_any();
	    any.insert_long(MEMB_INF_CTRL.value);
	    type_pr[4]=new Property(prop_ns,any);

	    prop_n=new NameComponent("org.omg.ft.Factories","string");
	    prop_ns=new NameComponent[1];
	    prop_ns[0]=prop_n;
	    any=orb.create_any();
	    FactoryInfosHelper.insert(any,fact_infos);
	    type_pr[5]=new Property(prop_ns,any);

	    prop_n=new NameComponent("org.omg.ft.FaultMonitoringGranularity","string");
	    prop_ns=new NameComponent[1];
	    prop_ns[0]=prop_n;
	    any=orb.create_any();
	    any.insert_long(Integer.parseInt(args[1]));
	    type_pr[6]=new Property(prop_ns,any);

	    prop_n=new NameComponent("org.omg.ft.FaultMonitoringInterval","string");
	    prop_ns=new NameComponent[1];
	    prop_ns[0]=prop_n;
	    any=orb.create_any();
	    any.insert_longlong((long)(Float.parseFloat(args[2])*1000.0));
	    type_pr[7]=new Property(prop_ns,any);

	    prop_n=new NameComponent("org.omg.ft.FaultMonitoringTimeout","string");
	    prop_ns=new NameComponent[1];
	    prop_ns[0]=prop_n;
	    any=orb.create_any();
	    any.insert_longlong((long)(Float.parseFloat(args[11])*1000.0));
	    type_pr[8]=new Property(prop_ns,any);

	    if(args[7].charAt(0)!='0'){
		prop_n=new NameComponent("org.omg.ft.CheckpointInterval","string");
		prop_ns=new NameComponent[1];
		prop_ns[0]=prop_n;
		any=orb.create_any();
		long per=1;
		try{
		    per=Integer.parseInt(args[7]);
		}catch(java.lang.NumberFormatException nfe){per=1;}
		any.insert_longlong(per*10000000);
		type_pr[9]=new Property(prop_ns,any);

		prop_n=new NameComponent("org.omg.ft.CheckpointPeriodic","string");
		prop_ns=new NameComponent[1];
		prop_ns[0]=prop_n;
		any=orb.create_any();
		type_pr[10]=new Property(prop_ns,any);
	    }

	    rm.set_type_properties(args[8],type_pr);

	    group_ref=rm.create_object(args[8],criteria,factory_cr_id);

	    if(group_ref==null){ System.out.println("Group reference is null, exit");return;}

	    String reference = orb.object_to_string(group_ref);
	    try
		{
		    java.io.FileOutputStream file = new java.io.FileOutputStream(args[3]);
		    java.io.PrintStream pfile=new java.io.PrintStream(file);
		    pfile.println(reference);
		    file.close();
		}
	    catch ( java.io.IOException ex )
		{
		    System.out.println("File error");
		}

	 	// HERE, A CHECK OF ARGUMENT 9 COULD BE DONE; if args[9].charAt(0)=='1' then the reference as to be registered in the name service as well

	}catch(Exception e){e.printStackTrace();}

    }
}
