/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: Statistics.java,v 1.0 2003/08/04 08:07:19 diasz Exp $
 *
 * Date         Author  Changes
 */

class LargeRec{
  long value;
  int nr_occ;

  public LargeRec(){
    value=-1;
    nr_occ=0;
  }
}
/** Class providing different utilities for computing the average of a list of values, standard deviations, to compute averages of
* values by removing n largest or n smallest values
* This file can be modified or extended to include more utilities
 * @author	<a href="mailto:diasz@ida.liu.se"> Diana Szentivanyi </a>
 * @version $Revision: 1.0 $ $Date:2003/06/23$
 */
public class Statistics{

/** Method used to compute the mean of a set of values given as an array of long
*/
    public static float mean(long[] data_set){
	long sum;
	int i;
	sum=0;
	for(i=0;i<data_set.length;i++)
	    sum+=data_set[i];
	return (float)((float)sum/(float)data_set.length);
    }

/** Method used to compute the mean of a set of values given as an array of long, from which the nr_largest
* largest values were removed
*/
  public static float mean_without_largest(long[] data_set, int nr_largest){
    LargeRec largest[]=new LargeRec[nr_largest];
    int i,j,length;
    long sum=0;
    for(i=0;i<nr_largest;i++)
      largest[i]=new LargeRec();
    for(i=0;i<data_set.length;i++){
      sum+=data_set[i];
      for(j=0;j<nr_largest;j++)
	if(data_set[i]>largest[j].value){

	  largest[j].value=data_set[i];
	  largest[j].nr_occ=1;
	}
      else
	if(data_set[i]==largest[j].value)
	  largest[j].nr_occ++;
    }
    length=data_set.length;
    for(i=0;i<nr_largest;i++){
	sum-=largest[i].nr_occ*largest[i].value;
	length-=largest[i].nr_occ;
    }

    float mean=(float)((float)sum/(float)length);
    return mean;
  }

/** Method used to obtain from an array of long a new array of long containing the values from the first array except the nr_largest
* largest values
*/

  public static long[] data_set_without_largest(long[] data_set, int nr_largest){
    LargeRec largest[]=new LargeRec[nr_largest];
    int i,j,k;
    long[] toRet;
    java.util.Vector temp=new java.util.Vector();
    boolean is_b,is_l;

    for(i=0;i<nr_largest;i++)
      largest[i]=new LargeRec();
    for(i=0;i<data_set.length;i++){
	is_b=false;
	for(j=0;j<nr_largest;j++)
	    if(data_set[i]>largest[j].value){
		if(largest[j].value>-1){
		    largest=shift_right(data_set[i],j,largest);
		}
		else{
		    largest[j].value=data_set[i];
		    largest[j].nr_occ=1;
		}
		is_b=true;
		break;
	    }
	if(!is_b)
	    for(j=0;j<nr_largest;j++)
		if(data_set[i]==largest[j].value)
		    largest[j].nr_occ++;
    }
    j=0;
    for(i=0;i<nr_largest;i++)
    for(i=0;i<data_set.length;i++){
      is_l=false;
      for(j=0;j<nr_largest;j++)
	if(data_set[i]==largest[j].value){
	  is_l=true;
	  break;
	}
      if(!is_l)
	temp.addElement(new Long(data_set[i]));
    }
    Long[] toRetL=new Long[temp.size()];
    toRetL=(Long[])temp.toArray(toRetL);
    toRet=new long[temp.size()];
    for(j=0;j<toRetL.length;j++)
      toRet[j]=toRetL[j].longValue();
    return toRet;
  }


/** Method used to compute the standard deiation of a set of values having the mean of the set
* @param	data_set the array of long values for which the st. dev. has to be computed
*			mean the mean of the data set
* @return 	the standard deviation for the data set
*/

    public static double st_dev(long[] data_set,float mean){
	double sum,mean_d=(double)mean;
	int i;
	sum=0.0;
	for(i=0;i<data_set.length;i++)
	    sum=sum+((double)data_set[i]-mean_d)*((double)data_set[i]-mean_d);
	sum=sum/((double)data_set.length-1.0);
	if(sum>0)
	    return Math.sqrt(sum);
	else
	    return -1;
    }

/** Method used to compute a percentage value given the value of the part and the value of the whole
*/

    public static double proc(double part, double whole){
	return (100.0*part/whole);

    }

    /** Method used to compute the vector that contains the values of the differences of corresponding values from two
    * vectors; for example {1,2} = diff({4,7},{3,5})
    */

    public static long[] diff(long[] data1, long[] data2){
	int i;
	long[] dif=new long[data1.length];
	for(i=0;i<data1.length;i++)
	dif[i]=data1[i]-data2[i];
	return dif;
    }

/** Method used to modify an array by shifting its content to the right, starting from a certian position where a new value will
* be inserted
* @param 	val	the value to insert
*			from the position from which to start the shift
*			x the array of values to be changed by hte shift
* @return	the changed array
*/
    public static LargeRec[] shift_right(long val,int from,LargeRec[] x){
	int i;
	for(i=x.length-1;i>from;i--){
	    x[i].value=x[i-1].value;
	    x[i].nr_occ=x[i-1].nr_occ;
	}
	x[from].value=val;
	x[from].nr_occ=1;
	return x;
    }

/** Method used to sum up the elements of an array of long
*/

    public static long sum(long[] data){
	int i;
	long s=0;
	for(i=0;i<data.length;i++)
	    s+=data[i];
	return s;
    }

/** Method used to read the set of values from a file and remember the values in an array of long
*/
    public static long[] read_data_file(String f_name){
	try{
	    java.util.Vector temp=new java.util.Vector();
	    long[] toRet;
	    int i;
	    java.io.RandomAccessFile d_file=new java.io.RandomAccessFile(f_name,"rw");
	    long length=d_file.length();
	    d_file.seek(0);
	    String reads;
	    while(d_file.getFilePointer()<length){
	      reads=d_file.readLine();
	      temp.addElement(Long.valueOf(reads));
	    }
	    Long[] toRetL=new Long[temp.size()];
	    toRetL=(Long[])temp.toArray(toRetL);
	    toRet=new long[toRetL.length];
	    for(i=0;i<toRet.length;i++)
	      toRet[i]=toRetL[i].longValue();
	    return toRet;
	}catch(java.lang.Exception e){e.printStackTrace();return null;}
    }
}
